<?php

namespace Drupal\advpoll\Form;

use Drupal\poll\Form\PollViewForm;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\BaseFormIdInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\poll\PollInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class ApprovalPollViewForm
 *
 * @package Drupal\advpoll\Form
 */
class ApprovalPollViewForm extends PollViewForm implements BaseFormIdInterface {

  /**
   * Index for the write in option.
   */
  protected const writeInIndex = -1;

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return 'approval_poll_view_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'approval_poll_view_form_' . $this->poll->id();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $view_mode = 'full') {
    // Get poll form with choice or results.
    $form = parent::buildForm($form, $form_state, $request, $view_mode);

    // Add poll-view-form class for AJAX selectors from the Poll module.
    $form['#attributes']['class'][] = 'poll-view-form';
    $form['#attributes']['class'][] = 'poll-view-form-' . $this->poll->id();

    // If we render a form (not results), upgrade it.
    if (isset($form['choice'])) {
      $pollType = '';
      if ($this->poll->hasField('field_poll_type')) {
        $pollType = $this->poll->field_poll_type->value;
      }

      // Change template.
      $form['#theme'] = 'poll_vote__advanced';

      // Upgrade for non-classic types.
      if ($pollType) {
        switch ($pollType) {
          case 'approval':
            // Set checkboxes.
            $form['choice']['#type'] = 'checkboxes';
            break;
        }
      }

      // Check write-in option.
      $isWriteInPoll = FALSE;
      if ($this->poll->hasField('field_writein')) {
        $isWriteInPoll = $this->poll->field_writein->value;
      }
      if ($isWriteInPoll) {
        // Add special option and the text field.
        $form['choice']['#options'][self::writeInIndex] = $this->t('Other (Write-in)');
        $form['write_in'] = [
          '#type' => 'textfield',
        ];

        if ($form['choice']['#type'] == 'checkboxes') {
          $form['write_in']['#states']['visible']['input[name="choice[' . self::writeInIndex . ']"]']['checked'] = TRUE;
        }
        else {
          $form['write_in']['#states']['visible']['input[name="choice"]']['value'] = self::writeInIndex;
        }
      }
      // Hide write-in choices.
      if (!empty($form['choice']['#options'])) {
        $choiceKeys = array_keys($form['choice']['#options']);

        /** @var \Drupal\poll\PollChoiceInterface[] $choicesWriteIn */
        $choicesWriteIn = \Drupal::entityTypeManager()->getStorage('poll_choice')->loadByProperties([
          'id' => $choiceKeys,
          'field_writein' => TRUE,
        ]);
        if ($choicesWriteIn) {
          foreach ($choicesWriteIn as $choice) {
            unset($form['choice']['#options'][$choice->id()]);
          }
        }
      }
    }

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function validateVote(array &$form, FormStateInterface $form_state) {
    parent::validateVote($form, $form_state);

    // Check multiple.
    if ($this->poll->hasField('field_number_of_votes') && !$this->poll->get('field_number_of_votes')->isEmpty()) {
      $choices = array_filter($form_state->getValue('choice'));
      $maxChoices = $this->poll->get('field_number_of_votes')->value;
      if ($maxChoices && count($choices) > $maxChoices) {
        $form_state->setErrorByName('choice', $this->t('Select up to @quantity @votes.', [
          '@quantity' => $maxChoices,
          '@votes' => $this->formatPlural($maxChoices, 'vote', 'votes'),
        ]));
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public function showResults(PollInterface $poll, FormStateInterface $form_state) {
    $showResults = parent::showResults($poll, $form_state);

    if (!$showResults) {
      // Check start date.
      if ($this->poll->hasField('field_start_date') && !$this->poll->get('field_start_date')->isEmpty()) {
        $startDate = \strtotime($this->poll->get('field_start_date')->value);
        if ($startDate > time()) {
          // Start date is in the future. Show results yet.
          $showResults = TRUE;
        }
      }
    }

    return $showResults;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * Cancel vote submit function.
   *
   * @param array $form
   *   The previous form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function cancel(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\poll\PollVoteStorageInterface $vote_storage */
    $vote_storage = \Drupal::service('poll_vote.storage');
    $vote_storage->cancelVote($this->poll, $this->currentUser());
    \Drupal::logger('poll')->notice('%user\'s vote in Poll #%poll deleted.', array(
      '%user' => $this->currentUser()->id(),
      '%poll' => $this->poll->id(),
    ));
    drupal_set_message($this->t('Your vote was cancelled.'));

    // In case of an ajax submission, trigger a form rebuild so that we can
    // return an updated form through the ajax callback.
    if ($this->getRequest()->query->get('ajax_form')) {
      $form_state->setRebuild(TRUE);
    }
  }

  /**
   * Save a user's vote submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function save(array $form, FormStateInterface $form_state) {
    $options = array();
    $choices = $form_state->getValue('choice');

    // Make array from single vote to use the same code.
    if (!is_array($choices)) {
      $choices = [$choices => $choices];
    }

    // Save vote.
    /** @var \Drupal\poll\PollVoteStorage $voteStorage */
    $voteStorage = \Drupal::service('poll_vote.storage');

    foreach($choices as $index => $choice) {
      if ($choice) {
        // Add write-in option if exists.
        if ($index == self::writeInIndex) {
          $chId = 0;

          $writeIn = $form_state->getValue('write_in');
          if ($writeIn) {
            // Check duplicate.
            $chId = array_search($writeIn, $this->poll->getOptions());
            if (empty($chId)) {
              // Create a new write-in option.
              $pollChoice = \Drupal::entityTypeManager()->getStorage('poll_choice')->create([
                'choice' => $writeIn,
              ]);
              // Add "write-in" option.
              $pollChoice->set('field_writein', TRUE);
              $pollChoice->save();

              $this->poll->get('choice')->appendItem($pollChoice);
              $this->poll->save();

              $chId = $pollChoice->id();
            }
          }

          if (empty($chId)) {
            continue;
          }
        }
        else {
          $chId = $index;
        }

        $options['chid'] = $chId;
        $options['uid'] = $this->currentUser()->id();
        $options['pid'] = $form_state->getValue('poll')->id();
        $options['hostname'] = \Drupal::request()->getClientIp();
        $options['timestamp'] = time();

        $voteStorage->saveVote($options);
      }
    }

    drupal_set_message($this->t('Your vote has been recorded.'));

    if ($this->currentUser()->isAnonymous()) {
      // The vote is recorded so the user gets the result view instead of the
      // voting form when viewing the poll. Saving a value in $_SESSION has the
      // convenient side effect of preventing the user from hitting the page
      // cache. When anonymous voting is allowed, the page cache should only
      // contain the voting form, not the results.
      $_SESSION['poll_vote'][$form_state->getValue('poll')->id()] = $form_state->getValue('choice');
    }

    // In case of an ajax submission, trigger a form rebuild so that we can
    // return an updated form through the ajax callback.
    if ($this->getRequest()->query->get('ajax_form')) {
      $form_state->setRebuild(TRUE);
    }

    // No explicit redirect, so that we stay on the current page, which might
    // be the poll form or another page that is displaying this poll, for
    // example as a block.
  }

}
